/*
 * flash.c: flashing user config and system option bytes
 *
 * by default, the pin used for PF2 is configured as a reset.
 * this code will check that the no-reset bit is set.
 * if it is not, this code will update the option bytes to
 * set the BOR voltage, enable BOR, and enable the no-reset bit.
 *
 * file creation: 20231019 0126
 */


#include "py32f0xx_conf.h"



/*
 * checks the flash option byte to determine if the reset pin
 * is configured to be PF2. if not, sets this bit as well as
 * the BOR values. if the bits were updated, they will be reloaded.
 * 
 * only call this function during startup, before interrupts
 * are enabled.
 * 
 * note: this code caused me to lose MCU. consider it buggy.
 */
void flash_optr_checkfix()
{
    FLASH_OBProgramInitTypeDef opt;    

    if ((FLASH->OPTR & FLASH_OPTR_NRST_MODE) == OB_RESET_MODE_RESET) {
        // let's update this
        opt.OptionType = OPTIONBYTE_RDP | OPTIONBYTE_USER;
        opt.USERType   = 0xff00;
        opt.USERConfig = OB_BOR_ENABLE  | OB_BOR_LEVEL_2p3_2p4 | OB_IWDG_SW | OB_WWDG_SW | OB_RESET_MODE_GPIO | OB_BOOT1_SYSTEM | OB_RDP_LEVEL_0;

        if (LL_FLASH_Unlock() != SUCCESS) return;
        if (LL_FLASH_OB_Unlock() != SUCCESS) return;
        
        LL_FLASH_OBProgram(&opt);

        LL_FLASH_OB_Lock();
        LL_FLASH_Lock();

        // reload newly programmed option bytes   
        LL_FLASH_OB_Launch();
    }
}

/*
 * manually load values into flash option byte, to configure
 * the BOR and configure the reset pin to be PF2.
 */
void flash_optr_set()
{
    FLASH->OPTR &= (FLASH_OPTR_BOR_EN_Msk | FLASH_OPTR_BOR_LEV_Msk | FLASH_OPTR_NRST_MODE_Msk);
    FLASH->OPTR |=  OB_BOR_ENABLE | OB_BOR_LEVEL_2p3_2p4 | OB_RESET_MODE_GPIO;
}

void flash_init()
{
    // load flash write timings for 8MHz operation
    // (these are listed in the datasheet)
    FLASH->TS0  = 0x3c;
    FLASH->TS2P = 0x3c;
    FLASH->TS3  = 0x3c;
    
    FLASH->TS1  = 0x90;
    FLASH->TPS3 = 0x240;

    FLASH->PERTPE  = 0x5dc0;
    FLASH->SMERTPE = 0x5dc0;
    FLASH->PRGTPE  = 0x1f40;
    FLASH->PRETPE  = 0x640;
}